## (一) 跨域

### (1) 跨域相关问题

#### 01 什么是跨域

**什么叫跨域**，浏览器有同源策略，只有当“协议”、“域名”、“端口号”都相同时，才能称之为是同源，其中有一个不同，即是跨域。

#### **02 同源策略的作用**是什么呢？

同源策略限制了从同一个源加载的文档或脚本如何与来自另一个源的资源进行交互。这是一个用于隔离潜在恶意文件的重要安全机制。

#### 03 我们又**为什么需要跨域**呢？

一是前端和服务器分开部署，接口请求需要跨域，二是我们可能会加载其它网站的页面作为iframe内嵌。

### **(2) 跨域的方法有哪些？**

1. jsonp跨域

2. cors跨域

   jsonp 只能支持 get 请求，cors 可以支持多种请求。cors是在服务器端设置, cors 并不需要前端做什么工作。

3. proxy(代理)跨域

   - nginx反向代理跨域
   - webpack代理跨域

4. websocket跨域

5. postMessage

6. 其它方式跨域

   - window.name + iframe
   - location.hash + iframe
   - document.domain 

### (3) 服务器端跨域

以nodejs的koa框架举例

```javascript
const cors = require('koa2-cors')
app.use(cors());
```

### (4) jsonp跨域

#### 01 jsonp问题

问题1: 为什么说jsonp不算真正的ajax?

1. ajax的核心是通过xmlHttpRequest获取非本页内容 jsonp的核心是动态添加script标签调用服务器提供的js脚本
2. jsonp只支持get请求，ajax支持get和post请求

问题2: **jsonp跨域原理是什么?**

​		尽管浏览器有同源策略，但是 `<script>` 标签的 src 属性不会被同源策略所约束，可以获取任意服务器上的脚本并执行。jsonp 通过插入script标签的方式来实现跨域，参数只能通过url传入，仅能支持get请求。

#### **02 json跨域步骤**

**服务器端让接口支持jsonp**

```javascript
const jsonp = require('koa-jsonp');
app.use(jsonp());
```

**前端代码**

1. 创建用户接收数据的方法,比如getData
2. 添加script标签, src地址指向了jsonp接口地址,带上参数callback=getData

```javascript
<body>
    
    <script>
        function getData(data) {
            console.log(data);
        }
    </script>
    <script src="http://localhost:3000/user?callback=getData"></script>
</body>
```

注意事项: jsonp接口跟普通接口不太一样, 需要服务器端做特殊处理